// ------------------------------------- Anfang Listing 1

class Behaelter
{ public:
   Behaelter(int size);
   virtual ~Behaelter();
   virtual void anfuegen(float element);
   virtual float& operator[](int position) const;
   virtual int anzahl() const;
  protected:
   int _groesse, _anzahl;
   float* _elemente;
};

class GepruefterBehaelter: public Behaelter
{ public:
   GepruefterBehaelter(int size);
   virtual void anfuegen(float element);
   virtual float& operator[](int position) const;
};

// Methoden von Behaelter ////////////////////////////

Behaelter::Behaelter(int groesse):
   _groesse(groesse),_anzahl(0)
{ _elemente = new float[_groesse];}

Behaelter::~Behaelter()
{ delete[] _elemente;}

void Behaelter::anfuegen(float element)
{ _elemente[_anzahl++] = element;}

float& Behaelter::operator[](int position) const
{ return _elemente[position];}

int Behaelter::anzahl() const
{ return _anzahl;}

// Methoden von GepruefterBehaelter //////////////////

GepruefterBehaelter::GepruefterBehaelter(int groesse):
  Behaelter(groesse)
{ if (_groesse < 1)
  throw "Behaelter muss mindestens 1 Element fassen.";
}

void GepruefterBehaelter::anfuegen(float element)
{ if (anzahl() == _groesse)
  throw "Ueberlauf.";
  else Behaelter::anfuegen(element);
}

float& GepruefterBehaelter::operator[](int position) const
{ if (position < 0 || position > anzahl())
  throw "Ungueltiger Index.";
  else return _elemente[position];
}

// ------------------------------------- Anfang Listing 2

class Fehlerpruefung; // schwebende Deklaration

class Behaelter
{ public:
   Behaelter(int groesse,Fehlerpruefung* pruefe);
   virtual ~Behaelter();
   virtual void anfuegen(float element);
   virtual float& operator[](int position) const;
   virtual int anzahl() const;
  protected:
   int _groesse, _anzahl;
   float* _elemente;
   Fehlerpruefung* _pruefe;
};

class Fehlerpruefung
{ public:
   virtual void groesse(int groesse) const;
   virtual void index(int position,int anzahl) const;
   virtual void ueberlauf(int anzahl,int groesse) const;
};

class KeinePruefung: public Fehlerpruefung
{ public:
   virtual void groesse(int groesse) const {};
   virtual void index(int position,int anzahl) const {};
   virtual void ueberlauf(int anzahl,int groesse) const {};
};

// Methoden von Behaelter ////////////////////////////

Behaelter::Behaelter(int groesse,Fehlerpruefung*
  pruefe = new Fehlerpruefung):_groesse(groesse),
  _pruefe(pruefe),_anzahl(0)
{ _pruefe->groesse(groesse);
  _elemente = new float[_groesse];
}

Behaelter::~Behaelter()
{ delete _elemente;
  delete _pruefe;
}

void Behaelter::anfuegen(float element)
{ _pruefe->ueberlauf(anzahl(),_groesse);
  _elemente[_anzahl++] = element;
}

float& Behaelter::operator[](int position) const
{ _pruefe->index(position,anzahl());
  return _elemente[position];
}

int Behaelter::anzahl() const
{ return _anzahl;}

// Methoden von Fehlerpruefung ///////////////////////

void Fehlerpruefung::groesse(int groesse) const
{ if (groesse < 1)
  throw "Behaelter muss mindestens 1 Element fassen.";
}

void Fehlerpruefung::index(int position,int anzahl) const
{ if (position < 0 || position > anzahl)
  throw "Ungueltiger Index.";
}

void Fehlerpruefung::ueberlauf(int anzahl,int groesse) const
{ if (anzahl == groesse)
  throw "Ueberlauf.";
}

// ------------------------------------- Anfang Listing 3

template <class Fehlerverhalten>
class Behaelter
{ public:
   Behaelter(int groesse);
   virtual ~Behaelter();
   virtual void anfuegen(float element);
   virtual float& operator[](int position) const;
   virtual int anzahl() const;
  protected:
   int _groesse, _anzahl;
   float* _elemente;
};

class Fehlerpruefung
{ protected:
   friend Behaelter<Fehlerpruefung>;
   static void groesse(int groesse);
   static void index(int position,int anzahl);
   static void ueberlauf(int anzahl,int groesse);
};

class KeinePruefung
{ protected:
   friend Behaelter<KeinePruefung>;
   static void groesse(int groesse);
   static void index(int position,int anzahl);
   static void ueberlauf(int anzahl,int groesse);
};

// Methoden von Behaelter ////////////////////////////

template <class Fehlerverhalten>
Behaelter<Fehlerverhalten>::Behaelter(int groesse):
  _groesse(groesse),_anzahl(0)
{ Fehlerverhalten::groesse(groesse);
 _elemente = new float[_groesse];
}

template <class Fehlerverhalten> inline
Behaelter<Fehlerverhalten>::~Behaelter()
{ delete _elemente;}

template <class Fehlerverhalten> inline
void Behaelter<Fehlerverhalten>::
  anfuegen(float element)
{ Fehlerverhalten::ueberlauf(anzahl(),_groesse);
  _elemente[_anzahl++] = element;
}

template <class Fehlerverhalten> inline
float& Behaelter<Fehlerverhalten>::
  operator[](int position) const
{ Fehlerverhalten::index(position,anzahl());
  return _elemente[position];
}

template <class Fehlerverhalten>
int Behaelter<Fehlerverhalten>::anzahl() const
{ return _anzahl;}

// Methoden von Fehlerpruefung ///////////////////////

inline void Fehlerpruefung::groesse(int groesse)
{ if (groesse < 1)
  throw "Behaelter muss mindestens 1 Element fassen.";
}

inline void Fehlerpruefung::index(int position,
  int anzahl)
{ if (position < 0 || position > anzahl)
  throw "Ungueltiger Index.";
}

inline void Fehlerpruefung::ueberlauf(int anzahl,
  int groesse)
{ if (anzahl == groesse)
  throw "Ueberlauf.";
}

// Methoden von KeinePruefung ////////////////////////

inline void KeinePruefung::groesse(int groesse) {};
inline void KeinePruefung::index(int position,
  int anzahl) {};
inline void KeinePruefung::ueberlauf(int anzahl,
  int groesse) {};

